package RFC::RFCCGI;
#$Id: RFCCGI.pm,v 1.50 2015/05/21 11:11:52 sqko Exp $
use strict;
use warnings;
use RFC::Header;
use RFC::Main;
use CGI qw(:standard :netscape);
require Exporter;

#################################################################################################
#
#
#  Package RFC::RFCCGI
#
# This module contains a number of utility functions for outputting properly formatted html code for
# user interface generation. Thus it mainly extends the CGI.pm module by Lincoln D. Stein.
# The module exports these functions in two groups.
#
# The :html group exports these functions:
#
# -- \begin{longtable}{ l p{10cm} }
# -- print_header($[\%]) & Prints the header information. Arguments: title. Any additional optional arguments (in the form of a hash) will be parsed along to the header() function supplied by CGI.pm. The functionautomatically appends a call to a javascript function logging users out after some time of no actions. \\
# -- print_end() & Prints the help button and ends the html output with the proper tag.  \\
# -- not_auth() & Prints the information supplied to the user if the user is nor authorised. Also prints a link to the log in page. \\
# -- xss($) & Wrapper for CGI::excapeHTML. \\
# -- print_hidden() & Prints a number of hidden fields used to maintain state. This includes user name and a cryptographic hash of the users password. \\
# -- print_hidden_rig() &  Same as print_hidden(), but with the additional information abut the active rig.\\
# -- logout() & Printys a logout button. \\
# -- action($) & Prints a hidden field with an action parameter with the specified value which can be used for program control flow.  \\
# -- RFC_start_html() & A wrapper for CGI::start_html. Any arguments (in the form of a hash) are passed to CGI::start_html. Automatically appends a reference to the javascript source file on the server. \\
# -- js_back() & Prints the javascript for gping backwards (uses the browser.back() fjavascrpt call). \\
# -- get_CGI_value($) & Retrieves the value of the specified CGI parameter (supplied by the web browser. \\
# -- get_CGI_value_clean($) & Same as get_CGI_value, but does pattern match on the retrieved value and only returns the part that matches. The pattern match is [{\backslash}w{\backslash}s{\backslash}.{\backslash},{\backslash}-]*. This has the benefit of untainting the returned parameter value (For taint checks in perl and web access, refer Lincoln D. Steins book \textit{Official Guide to Programming with CGI.pm}) \\
# -- rig_auth() & This function returns 1 if the user is certified for the rig in question. It only checks if the rig in question is mentioned in the 'safety_task_access' section in the global configuration Where each rig is supposed to define which taskid(s) are connected with which rig. If more than one taskid is connected with the rig (specified by a list of id numbers), the user has to be authentificated for all taskids in the list for authentification to occour. The definition is in the form:  rig1 = 45,1 where 45 and 1 are the taskids to check against. Usage is either with named rig argument or without any argument, if used without argument, the current rig number is used as default. Usage: rig_auth(), rig_auth($rig) or rig_auth($rig,$taskidstring). If the last call method is used the taskidstring must be a string of integers seperated by comma. \\
# -- rig_cert() & This function returns 1 if the user is certified for the rig in question. It only checks if the rig in question is mentioned in the 'safety_task_access' section in the global configuration Where each rig is supposed to define which taskid(s) are connected with which rig. If more than one taskid is connected with the rig (specified by a list of id numbers), the user has to be certified for the first in the list for certification to occour. The definition is in the form: rig1 = 45,1 where 45 is the taskid to check against. Usage is either with named rig argument or without any argument, if used without argument, the current rig number is used as default. Usage: rig_cert() or rig_cert($rig). \\
# --\end{longtable}
#
#
# The :cgi group of functions include the following:
#
# --\begin{longtable}{ l p{10cm} }
# -- get_CGI_value($) & See above. \\
# -- get_CGI_value_clean($) &  See above. \\
# -- rig_cert() & See above \\
# -- rig_auth() & See above \\
# -- action($) & See above \\
# -- not_auth() & See above \\
# -- xss($) & See above \\
# -- login_ok() & Checks if the user supplied login credentials are ok. This can be either against a small local database, or against a full RDBMS. \\
# -- menu_button(@) & Prints a menu button. Arguments: name, value, style. The name will be the CGI parameter name, the value vill be the text on the button and the style is a style class name to use for displaying. \\
# -- create_menu_field & Prints the html tags to create a menu field.  \\
# -- top_nav_bar_start() & Prints the html tags to start the top navigation bar (table specifications etc.) \\
# -- top_no_button() & Prints a no action button (goes nowhere) in the top navigation bar. \\
# -- top_nav_bar_button() & prints a top navigation button. Arguments: File, name, value, style, [optional additional name, value and force triplets]. The file is the cgi-script to be called upon button press, the name,value and style arguments are passed to menu_button() and the additional optional arguments are used to initialise and print hidden html fields in the form of name-value pairs and a force argument (1 for force value, 0 for allow reuse of value). \\
# -- tab_newrow() & Prints a new row in the top navigation bar. \\
# -- top_js_return() & Prints a top navigation return button (uses the javascript printed by js_back(), see above) \\
# -- end_top_bar() & Prints the end of the top navigation bar. \\
# -- is_logout_time() & Returns 1 or 0 depending on time since last action by the current user (used for authentification purposes). Deprecated from version 4.6.2 onwrads!\\
# -- is_admin() & Return 1 if the current user is an administrator (gets explicit access) \\
# -- user_auth() & Returns 1 if the user is authorised for the rig in question. Some of the authentification is forwarded to rig_auth() \\
# -- get_user() & Returns the username of current user. \\
# -- get_pass() & Returns the cryptographic hash of the current users password. \\
# -- logout_delay() & returns the number of minutes a user can remain inactive before automatic logout occours. \\
# --\end{longtable}
#
#
# An additional function which is often used (but which is not exported by default) is the
# print_js_validate() function. 
# This function prints a javascript function for client side input validation.
# The function can check any number of fields (names specified in the arguments list) 
# and can either check if a field is filled out, OR check that if a field value is 'NEW' 
# then an other field must not be empty The syntax for this case is: 'name_1#name_2' where
# name_1 is the name of the field which can be 'NEW' and name_2 is the name of field 2 
# which must not be empty. Another form of check is where a number of fields      
# exists and where not all fields can be empty to use this option use the ':' separator
# in the string: '$field_group_name:$field1:$field2:...'.
#
# Usage:  print_js_validate($form_name,[$field_names...]) It is also possible to include 
# regular expression tests in the validation. To do this use the format: 
# 'name;"regexp";[opt $]Error message'. If the $ is the value of the input field is placed 
# in front of the errormessage!                                           
#
#
#################################################################################################



BEGIN {
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
    $VERSION = 1.0;
    @ISA = qw(Exporter);
    @EXPORT = qw($homedir $userdir);
    @EXPORT_OK = qw($name top_nav_bar_button print_header print_end not_auth print_hidden logout login_ok action get_user RFC_start_html print_id js_back get_CGI_value get_CGI_value_clean js_back login_ok menu_button not_auth create_menu_field top_no_button tab_newrow action end_top_bar top_nav_bar_start top_js_return create_menu_field is_logout_time get_id is_admin user_auth get_user get_pass print_hidden_rig rig_cert rig_auth xss ssx);
    %EXPORT_TAGS = (
		    html => [ qw($name print_header print_end not_auth print_hidden print_hidden_rig logout action RFC_start_html js_back get_CGI_value get_CGI_value_clean rig_cert rig_auth xss) ],
		    cgi => [ qw(login_ok get_CGI_value get_CGI_value_clean menu_button create_menu_field top_no_button tab_newrow action end_top_bar top_nav_bar_start top_js_return top_nav_bar_button not_auth is_logout_time get_id is_admin user_auth get_user get_pass rig_cert rig_auth xss) ],
                    );
}

our $name = $RFC::Header::COPYRIGHT.' fuel cell test facility';

my $msg = '';

return 1;

sub xss($)
################################################################
#### Quote the supplied string to remove potential dangerous ###
#### XSS markup                                              ###
#### Returns a sanitized version of the supplied string      ###
################################################################
{
    my $str = shift;
    return escapeHTML($str);
}

sub ssx($)
################################################################
#### Inverts any xss sanitation (nescesarry for database     ###
#### searches                                                ###
#### Returns a usanitized version of the supplied string     ###
################################################################
{
    my $str = shift;
    return CGI::unescapeHTML($str);
}


sub is_logout_time
######################################################################
### Function which returns 1 if the login timeout time is exceeded ###
### If $use_time = 0 the function always returns 0                 ###
######################################################################
### Deprecated from 4.6.2 onwards                                  ###
######################################################################
{
    my $time = &time_string(&get_time("now"));
    my $logouttime = 0;
    $logouttime = param('time');
    my $res = 0;
    if ($time > $logouttime)
    {
        return 1;
    }
    return 0;
}

sub logout_delay()
{
    return &passwd_client('logout_delay');
}

sub login_ok()
#################################################################
### this function checks if the user and password obtained    ###
### from the web request is in order and that not too long    ###
### since last action has occured                             ###
#################################################################
{
    return &user_ok(@_)
}

sub user_ok()
#################################################################
### this function checks if the user and password obtained    ###
### from the web request is in order (CGI_wraper for user_ok) ###
### Any arguments is passed along to get_id()                 ###
#################################################################
{
    my ($user,$pass) = &get_id(@_);
    my $res = &passwd_client('checkuser',$user,$pass);
    chomp $res if (defined($res));
    $res = 0 if (!defined($res) || $res =~ /error/i || $res < 0);
    return $res;
}

sub user_auth($$)
#######################################################
### CGI-wrapper for user_auth, gets the username by ###
### calling get_user.                               ###
### Usage: user_auth($table,$mode)                  ###
#######################################################
{
    my $tab = shift;
    my $mode = shift;
    my $m = 0;
    $m = 1 if ($mode eq 'r');
    $m = 2 if ($mode eq 'c');
    $m = 3 if ($mode eq 'm');
    my ($u,$p) = &get_id();
    my $res = &passwd_client('checkauth',$u,$p,$tab,$m);
    chomp $res;
    $res = 0 if (!defined($res) || $res < 0);
    return $res;
}

sub is_admin()
##################################################################
### this function checks if the user and password obtained     ###
### from the web request is in order (CGI_wraper for is_admin) ###
### Any arguments is passed along to get_user()                ###
##################################################################
{
    my ($user,$pass) = &get_id(@_);
    chomp $user;
    my $res = &passwd_client('isadmin',$user,$pass);
    chomp $res;
    $res = 0 if (!defined($res) || $res < 0);
    return $res;
}

sub is_user($)
##################################################################
### this function checks if the user already exsists           ###
### Is a client-server wrapper for is_user_server()            ###
##################################################################
{
    my $user = shift;
    chomp $user;
    my $res = &passwd_client('isuser',$user);
    chomp $res;
    return $res;
}

sub get_email()
################################################################
#### This function is a wrapper for get_email() where the    ###
#### user is found from get_user()                           ###
################################################################
{
    my $user = &get_user();
    my $res = &passwd_client('email',$user);
    chomp $res;
    return $res;
}


sub rig_auth()
##########################################################
### This function returns 1 if the user is authorised  ###
### to access the rig in question. It only checks if   ###
### the rig in question is mentioned in the 'safety_   ###
### task_access' section in the global configuration   ###
### Where each rig is supposed to define which taskid  ###
### is connected with which rig.                       ###
### in the form:  rig1 = 45    where 45 is the taskid  ###
### Usage is either with named rig argument or without ###
### any argument, if used without argument, the        ###
### current rig number is used as default.             ###
### Usage: CGI_rig_auth() or CGI_rig_auth($rig)        ###
##########################################################
{
    my $rig = &get_CGI_value_clean('rig');
    if (@_)
    {
	my $r = shift;
	if (ref($r) eq 'RFC::Rig')
	{
	    $rig = $r->name;
	}
	else
	{
	    $rig = $r;
	}
    }
    $rig =~ s/[\r\n]*//g if (defined($rig));
    my $taskid = &get_cv('safety_task_access','rig'.$rig);
    $taskid = shift if (@_);
    return 1 unless (defined($taskid) && $taskid);
    return 1 if ($taskid =~ /NONE/); ## Specproc for override ###
    my $user = &get_user();
    my $retval = 1;
    ### Run through all taskids in list and return 1 if all is ok ###
    foreach my $tid (split(/\s*\,\s*/,$taskid))
    {
	my $res = &passwd_client('check_task_status',$user,$tid);
	if (!defined($res) || $res < 0)
	{
	    &errorlog("Check_task_status for task $tid did not return expected value: returned: $res instead!");
	    return 0;
	}
	$retval = 0 unless ($res);
    }
    return $retval;
}


sub rig_cert()
##########################################################
### This function returns 1 if the user is certified   ###
### for the rig in question. It only checks if         ###
### the rig in question is mentioned in the 'safety_   ###
### task_access' section in the global configuration   ###
### Where each rig is supposed to define which taskid  ###
### is connected with which rig.                       ###
### in the form:  rig1 = 45    where 45 is the taskid  ###
### Usage is either with named rig argument or without ###
### any argument, if used without argument, the        ###
### current rig number is used as default.             ###
### Usage: CGI_rig_auth() or CGI_rig_auth($rig)        ###
##########################################################
{
    my $rig = &get_CGI_value_clean('rig');
    if (@_)
    {
	my $r = shift;
	if (ref($r) eq 'RFC::Rig')
	{
	    $rig = $r->name;
	}
	else
	{
	    $rig = $r;
	}
    }
    $rig =~ s/[\r\n]*//g if (defined($rig));
    my $taskid = &get_cv('safety_task_access','rig'.$rig);
    return 1 unless (defined($taskid) && $taskid);
    return 1 if ($taskid =~ /NONE/); ## Specproc for override ###
    my $user = &get_user();
    my @tlist = split(/\s*\,\s*/,$taskid);
    ### Only check the first item in the taskid list for certification ###
    my $ok = &passwd_client('check_task_cert',$user,$tlist[0]);
    return $ok unless ($ok < 0);
    &errorlog("Check_task_status did not return expected value: returned: $ok instead!");
    return 0;
}


sub rig_mode()
##########################################################
### CGI_rig_mode returns 1 or 0 depending on if        ###
### the rig in question is mentioned in the 'safety_   ###
### task_access' section in the global configuration   ###
### Where each rig is supposed to define which taskid  ###
### is connected with which rig.                       ###
### in the form:  rig1 = 45    where 45 is the taskid  ###
### Usage is either with named rig argument or without ###
### any argument, if used without argument, the        ###
### current rig number is used as default.             ###
### Usage: CGI_rig_auth() or CGI_rig_auth($rig)        ###
##########################################################
{
    my $rig = &get_CGI_value_clean('rig');
    if (@_)
    {
	my $r = shift;
	if (ref($r) eq 'RFC::Rig')
	{
	    $rig = $r->name;
	}
	else
	{
	    $rig = $r;
	}
    }
    $rig =~ s/[\r\n]*//g if (defined($rig));
    my $taskid = &get_cv('safety_task_access','rig'.$rig);
    return 1 if ($taskid);
    return 0;
}

sub get_id()
##########################################################
### This function returns the user and password as an  ###
### array where the username is the first element and  ###
### the password is the second element                 ###
### usage:  my ($user,$pass) = &get_id;                ###
### Any arguments is passed to get_user and get_pass   ###
##########################################################
{
    my @l = ();
    push @l, &get_user(@_);
    push @l, &get_pass(@_);
    return @l;
}

sub get_users()
{
    my ($u,$p) = &get_id();
    my $res = &passwd_client('get_users',$u,$p);
    if (defined($res))
    {
	chomp $res; 
	return split(/\n+/,$res);
    }
    return undef;
}

sub get_user()
###########################################################
### This function gets the current user                 ###
### Returns undef if the name parameter does not exsis  ###
### if an argument is given, the function first checks  ###
### for a parameter and then for cookies, revers if not ###
###########################################################
{
    return param('name');
}

sub get_pass()
###########################################################
### This function gets the current passwd               ###
### Returns undef if the name parameter does not exsis  ###
### if an argument is given, the function first checks  ###
### for a parameter and then for cookies, revers if not ###
###########################################################
{
    return param('pass');
}

sub action($)
#############################################################
### This function prints a hidden field with hame action ####
### and value equal to the argument                      ####
### Usage  action($value);                               ####
#############################################################
{
    print hidden(-name=>'action',-value=>shift,-force=>'1');
}

sub login
################################################
### This function prints a login table      ####
### The html tags created is:               ####
### name: the username                      ####
### pass: The password                      ####
### action: The name of the submit button   ####
### The value of this is 'Login'            ####
################################################
{
    print "\n",'<script lanuage="javascript">',"\n";
    my $script = 'popup.html';
    if (get_config_value('global','use_passwd') && get_config_value('global','use_passwd') =~ /yes/)
    {
	$script = 'popup_pwd.html';
    }
    print <<END;

    function windowOpen() {
        var myWindow=window.open('../../celltest/$script','windowRef','width=450,height=100,left=300,top=200');
        myWindow.location.href = '../../celltest/$script';
    if (!myWindow.opener)
        myWindow.opener = self;
        myWindow.focus();
    }

    function go()
    {
	document.login.jscheck.value = 1;
        document.login.action.click();
    }

    windowOpen();
//--></script>

END

    print start_form(-name=>'login',-class=>"form",-onsubmit=>'return checkjs()',-action=>'login.cgi');
    print hidden(-name=>'jscheck',-value=>'0',-force=>'1');
    print hidden(-name=>'name',-value=>'',-value=>'1');
    print hidden(-name=>'pass',-value=>'',-value=>'1');
    print submit(-name=>'action',-value=>'Login',-class=>'hidden');
    print '</th></tr></table>';
    print end_form;
}

sub RFC_start_html
#####################################################################
### This function is a wrapper for start_html but automatically #####
### if the onload argument is already given the logout script   #####
### is just added to the functions which are to be run.         #####
#####################################################################
{
    my %h = @_;
    my $onload = 0;
    for my $key (keys %h)
    {
        if ($key =~ /-onload/i)
        {
            my $str = $h{$key};
            my $e = '';
            if ($str =~ /([\'\"])$/)
            {
                $e = $1;
                $str =~ s/[\'\"]$//;
            }
            $str = $str . ';'.$e;
            $h{$key} = $str;
            $onload = 1;
        }
    }
    $h{'-script'} = { -language => 'javascript', -src => '/celltest/RFCcontrol.js' };
    return CGI::start_html(%h);
}

sub print_header($)
#########################################################
### This function prints the header for the web pages ###
### Any second argument is used as the javascript     ###
### function to be called after the page loads        ###
#########################################################
{
    my $time = &logout_delay();
    $time =~ /(\d+)/;
    $time = $1+0;
    $time = ($time * 60 * 1000) - 10000;
    my $tit = shift;
    my $use_time = 1;
    $use_time = 0 if (get_config_value('global','use_time') && get_config_value('global','use_time') =~ /no/i);
    my $res = header(-charset=>'utf-8');
    my %h = @_;
    my $onload = 0;
    for my $key (keys %h)
    {
        if ($key =~ /-onload/i)
        {
            my $str = $h{$key};
            my $e = '';
            if ($str =~ /([\'\"])$/)
            {
                $e = $1;
                $str =~ s/[\'\"]$//;
            }
            $str = $str . ';doAlertLogout('.$time.');'.$e if ($use_time);
            $h{$key} = $str;
            $onload = 1;
        }
    }
    if ($use_time && !$onload)
    {
        $res .= RFC_start_html(%h,-title=>$tit,-style=>{src=>'/celltest/RFCcontrol.css'},-onLoad=>'doAlertLogout('.$time.')');
    }
    else
    {
        $res .= RFC_start_html(%h,-title=>$tit,-style=>{src=>'/celltest/RFCcontrol.css'});
    }
    print $res;
    print "\n\n";
    print start_form(-name=>'login_info');
    print_hidden();
    print hidden(-name=>'time',-value=>0,-force=>1);
    print hidden(-name=>'jscheck',-value=>0,-force=>1);
    my $rig = param('rig');
    if ($rig)
    {
	$rig += 0;
	print hidden(-name=>'rig',-value=>$rig,-force=>1) if ($rig);
    }
    print end_form();
    print "\n\n";
}

sub print_end()
#######################################
### Prints the end_html command #######
#######################################
{
    my $file = 'manual.pdf';
    $file = shift if (@_);
    print "\n\n";
    print p();
    print center(a({-href=>'javascript:load_help();'},'Help'));
    print <<END;

<script lanuage="javascript">
function load_help()
{
    var myWindow=window.open('/celltest/$file','windowRef','width=700,height=500,left=100,top=100,resizable=yes,scrollbars=yes');
    myWindow.location.href = '/celltest/$file';
    if (!myWindow.opener)
    {
        myWindow.opener = self;
        myWindow.focus();
    }
}

</script>

END
    print center(p('Copyright &copy '.$RFC::Header::COPYYEAR.' '.$RFC::Header::IMPLEMENTERS.', '.$RFC::Header::COPYRIGHT));
    print end_html;
}

sub logout
##########################################
###  Function printing a logout button ###
##########################################
{
    my $user = param('user');
    print start_form(-action=>'prelogin.cgi');
    if ($user)
    {
	print submit(-class=>'s0',-name=>'logout',-value=>'Logout '.$user);
    }
    else
    {
	print submit(-class=>'s0',-name=>'logout',-value=>'Logout');
    }
    print end_form;
}


sub not_auth()
{
    print_header("Access to $name denied!");
    print h1({-class=>'C'},"Access to $name denied!");
    if ($msg)
    {
	print h2({-class=>'C'},$msg);
    }
    else
    {
	print h2({-class=>'C'},"Possibly wrong username");
    }
    print '<center>';
    &create_menu_field;
    print start_form(-class=>"form",-name=>'Return',-action=>'prelogin.cgi');
    print '<table>';
    print '<tr><td width=200>';
    &menu_button("To login page","To login page",'s4');
    print '</td></tr></table>';
    print end_form();
    print '</center>';
    print_end;
}


sub js_back
##############################################
### This function prints a javascript which ##
### takes the browser one step backwards    ##
##############################################
{
    print "\n",'<script>',"\n";
    print 'function back() {',"\n";
    print '  history.back();',"\n";
    print '  return false;',"\n";
    print '}',"\n";
    print '</script>',"\n";
}

sub print_hidden()
#########################################################################
### Prints the logininformation to be transfered to the next cgi call ###
### If called with arguments the username and password is used from   ###
### the passed arguemnts instead from CGI parameters                  ###
#########################################################################
{
    my $user = get_user();
    my $pass = get_pass();
    if (@_)
    {
	$user = shift;
	$pass = shift;
    }
    print hidden(-name=>'name',-value=>$user,-force=>1);
    print hidden(-name=>'pass',-value=>$pass,-force=>1);
}

sub print_hidden_rig()
#########################################################################
### Prints the logininformation to be transfered to the next cgi call ###
### If called with arguments the username and password is used from   ###
### the passed arguemnts instead from CGI parameters                  ###
#########################################################################
{
    &print_hidden(@_);
    print hidden(-name=>'rig');
}


sub top_nav_bar_start
##################################################
###Function creating the top_navigator_bar   #####
###                                          #####
##################################################
{
    print '<table /*border="2px"*/ class="top_table", width=100% >';
    print '<col width=20%", span="5">';
    print '<tr>';
    &top_logout();
    print '</td>';
    &print_page();
}

sub print_page()
###################################################
### Function creating a top bar button for      ###
### printing the active page                    ###
###################################################
{
    print '<td>';
    print "\n<script lanuage=\"javascript\">\n";
    print "function printWindow() {\n";
    print "   var bV = parseInt(navigator.appVersion);\n";
    print "   if (bV >= 4) {\n";
    print "      window.print();\n";
    print "   }\n";
    print "}\n";
    print "</script>\n";
    print start_form(-name=>'dummy',-class=>'form');
    print button(-name=>'Printpage',-value=>'Print page',-class=>'print',-onclick=>'printWindow()');
    print end_form;
    print '</td>';
}


sub top_nav_bar_button($$@)
#######################################################################
### Prints a nav bar button which when pressed                     ####
### takes the user to the $file CGI-script and                     ####
### parses the action parameter $action to the                     ####
### script. For the 3 last arguments, see                          ####
### menu_button()                                                  ####
### Usage top_nav_bar_button($file,$action,$1,$2,$3,[opt args])    ####
### Arguments 1 to 3 are parced directly to                        ####
### menu_button().                                                 ####
### It is possible to specify parameters to be printed in hidden   ####
### fields by adding additional arguments, each additional         ####
### argument must be a comma separated list specifying (in order!) ####
### name, value and force (0/1). if the parameter is just to be    ####
### 'sticky, let the last two elements be blank, eks: search_val,, ####
### would print a hidden field with name 'search_val' and the      ####
### value would be whatever the previous field with name           ####
### 'searh_val' had.                                               ####
#######################################################################
{
    my $file = shift;
    my $action = shift;
    &create_menu_field;
    if ($file)
    {
        print start_form(-action=>$file,-class=>"form");
    }
    else
    {
        print start_form(-class=>"form");
    }
    &action($action) if ($action);
    my @args = ();
    push @args, shift @_;
    push @args, shift @_;
    push @args, shift @_;
    &menu_button(@args);
    &print_hidden();
    while (@_)
    {
        my @s = split(/\s*\,\s*/,shift @_);
        print hidden(-name=>$s[0],-value=>$s[1],-force=>$s[2]);
    }
    print end_form();
    print '</td>';
}



sub top_no_button
#################################
### Prints a 'empty' button   ###
### in the top nav bar        ###
#################################
{
    &create_menu_field;
    start_form;
    &menu_button('','','none');
    end_form;
    print '</td>';
}

sub top_js_return
#############################################################
### Prints a return button  in the top navbar which uses  ###
### javascript to bring the user one step back            ###
### Note that the function requires that the function     ###
### &js_back() is used somewhere on the page!!!           ###
#############################################################
{
    &create_menu_field;
    print start_form(-class=>"form",-onsubmit=>"return back()");
    &print_hidden();
    &menu_button('Back','Back','back');
    print end_form();
    print '</td>';
}
sub top_logout
#############################################################
### Prints a logout button  in the top navbar which uses  ###
### javascript to ask if the user wants to logout         ###
#############################################################
{
    my  $name = get_user();
    my $scr = '/cgi-bin/celltest/ajax.cgi';
    print "\n\n<script lanuage=\"javascript\">\n";
    print "IsLoggedIn = 1;\n\n";
    print "function LogOut() {\n";
    print "   if (IsLoggedIn == 0) {\n";
    print "       raw_ajax('$scr','?action=logout&name=$name',doLogOut);\n";
    print "       return true;\n";
    print "   }\n";
    print "   if (confirm('Do you want to log out?')) {\n";
    print "       raw_ajax('$scr','?action=logout&name=$name',doLogOut);\n";
    print "       return true;\n";
    print "   }\n";
    print "   return false;\n";
    print "}\n";
    print "function doLogOut(res) {\n";
    print "   if (IsLoggedIn == 1) {\n";
    print "       alert(res);\n";
    print "   }\n";
    print "   document.logout.submit();\n";
    print "   return true;\n";
    print "}\n";
    print "</script>\n\n";
    my $user = param('user');
    &create_menu_field;
    print start_form(-class=>"form",-name=>'logout',-action=>'prelogin.cgi');
    print hidden(-name=>'dummy',-value=>0,-force=>1);
    if ($user)
    {
	print button(-name=>'Logout',-value=>'Logout '.$user,-class=>'s0',-onclick=>"return LogOut()");
    }
    else
    {
	print button(-name=>'Logout',-value=>'Logout',-class=>'s0',-onclick=>"return LogOut()");
    }
    print end_form();
    print '</td>';
}


sub tab_newrow
################################
### prints a html collumn    ###
### separator                ###
################################
{
    print '</tr><tr>';
}

sub end_top_bar
#################################
### Ends the top nav bar     ####
#################################
{
    print '</tr>';
    print '</table>';
}


sub create_menu_field
###################################################
### This function prints a table field with a   ###
### specific height                             ###
###################################################
{
    print '<td height="30px", margin="0px", padding="0px">';
}

sub menu_button($$$)
##################################################################
### This funtion creates a menu button                         ###
### Usage: menu_button($bgcolor,$color,$value,$name);          ###
##################################################################
{
    my $name = shift;
    my $value = shift;
    my $style = shift;
    print submit(-name=>$name,-class=>$style,-value=>$value);
}

sub clean($)
###########################################################
#### This function cleans a string for all special      ###
#### characters (only allowed ones are '-','.' and ',') ###
###########################################################
{
    my $val = shift;
    my $ret = undef;
    if (defined($val) && $val =~ /([\w\s\.\,\-]*)/)
    {
        $ret = $1;
    }
    return $ret;
}

sub get_CGI_value_clean($)
##################################################
#### Wrapper for &clean and &get_CGI_value    ####
##################################################
{
    my $id = shift;
    my $val = &get_CGI_value($id);
    return &clean($val);
}

sub get_CGI_value($)
####################################################
#### This function gets the CGI parameter with   ###
#### $name, returns undef if the parameter is    ###
#### not defined in the input parameters         ###
#### Usage: get_CGI_value($name)                 ###
####################################################
{
    my $id = $_[0];
    my $val = undef;
    $val = param($id) if defined(param($id));
    return $val;
}


sub get_config_value(@)
{
    return &RFC::Main::get_cv(@_);
}

sub passwd_client(@)
{
    return RFC::Main::passwd_client(@_);
}

sub get_logout_time()
#########################################################
### Returns the maximum time before automatic logout ####
#########################################################
{
    my $logoutdelay = &RFC::Main::get_cv('global','logoutdelay');
    my $time = &time_string(&get_time($logoutdelay));
    return $time;
}


sub get_time($)
##########################################################################
### Function which returns the time in format YYYY MM DD HH MM SS      ###
### Output is in the form of an array                                  ###
### Input is either 'now' or the offset (check 'man date' for details) ###
##########################################################################
{
    my $t = shift;
    my $date_format = " +\"%Y:%m:%d:%H:%M:%S\"";
    my $date = "--date=\"$t\"";
    my $time_format = "$date_format $date\n";
    my $time = `/bin/date $time_format`;
    my @t = split(/:/,$time);
    return @t;
}

sub time_string(@)
##################################################################
### Function which returns a string with the concatanated date ###
### information. Places leading 0'es where nescesary           ###
##################################################################
{
    my $time ="";
    my @t;
    my @l = @_;
    foreach (@l)
    {
	if ($_ < 10)
	{
	    $_ = '0'.$_ unless (/0\d/);
	}
    }
    return join('',@l);
}

sub print_js_validate
###############################################################
### This function prints a javascript for input validation  ###
### The function can check any number of fields (names      ###
### specified in the arguments list) and can either check   ###
### if a field is filled out, OR check that if a field      ###
### value is 'NEW' then an other field must not be empty    ###
### The syntax for this case is: 'name_1#name_2' where      ###
### name_1 is the name of the field which can be 'NEW' and  ###
### name_2 is the name of field 2 which must not be empty.  ###
### Another form of check is where a number of fields       ###
### exists and where not all fields can be empty            ###
### to use this option use the ':' separator in the string: ###
###   '$field_group_name:$field1:$field2:...'               ###
### Usage:  print_js_validate($form_name,[$field_names...]) ###
### It is also possible to include regular expression tests ###
### in the validation. To do this use the format:           ###
### 'name;"regexp";[opt $]Error message'. If the $ is       ###
### the value of the input field is placed in front of the  ###
### errormessage!                                           ###
###############################################################
{
    my $name = shift;
    ### If the name argument begins with a '#' then the initial  ###
    ### part f the function has already been added (likely       ###
    ### py print_js_validate_modify() ) and is ignored and the   ###
    ### '#' in the name is removed                               ###
    if ($name =~ /\#(\w+)/)
    {
        $name = $1;
    }
    else
    {
        print "\n\n";
        print '<script>',"\n";
        print 'function checkinput() {',"\n";
        print '  var f = document.',$name,";\n";
    }
    print "  var txt = '';\n";
    print "  var newf = '';\n";
    while (scalar(@_) > 0)
    {
        my $s = shift;
        if ($s =~ /^\w+$/)
        {
            ############ Print an normal field check ########
            print '  txt = f.'.$s.'.value;',"\n";
            print '  if (txt == null || txt.length == 0) {',"\n";
            print '     alert("Required field \"'.$s.'\" has not been input!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
        }
        if ($s =~ /^(\w+)\;\"(.*)\"\;([\$\w\s\,\\\-\+]+)$/)
        {
            my $name = $1;
            my $regexp = $2;
            my $text = $3;
            ############ Print an normal field check with regexp ########
            print '  txt = f.'.$name.'.value;',"\n";
            print '  if (txt == null || txt.length == 0) {',"\n";
            print '     alert("Required field \"'.$name.'\" has not been input!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
            print '  reg = /'.$regexp."/;\n";
            print '  if (!reg.test(txt)) {',"\n";
            if ($text =~ /\$/)
            {
                $text =~ s/\$//g;
                print '     alert(txt+"'.$text.'");',"\n";
            }
            else
            {
                print '     alert("'.$text.'");',"\n";
            }
            print '     return false;',"\n";
            print '  }',"\n";
        }
        if ($s =~ /^(\w+)\#(\w+)$/)
        {
            ############ Prints a composite field check (NEW check) #######
            print '  txt = f.'.$1.'.value;',"\n";
            print '  if (txt == null || txt.length == 0) {',"\n";
            print '     alert("Required field \"'.$1.'\" has not been input!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
            print '  newf = f.'.$2.'.value;',"\n";
            print '  if (txt == "NEW" && (newf == null || newf.length == 0)) {',"\n";
            print '     alert("When using \"NEW\" in drop-down box \"'.$1.'\" new field must not be empty!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
        }
        if ($s =~ /^(\w+)\=(\w+)/)
        {
           ############ Prints a composite field check (yes check) #######
            print '  txt = document.'.$name.'.'.$1.'.value;',"\n";
            print '  newf = f.'.$2.'.value;',"\n";
            print '  if (txt == "Yes" && (newf == null || newf.length == 0)) {',"\n";
            print '     alert("When using \"Yes\" in drop-down box \"'.$1.'\" field \"'.$2.'\" must not be empty!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
            if ($s =~ /\=\w+\;\"(.*)\"\;([\$\w\s\,\\\-\+]+)$/)
            {
                print '  else {',"\n";
                print '     if (txt == "Yes" ) {',"\n";
                my $regexp = $1;
                my $text = $2;
                ############ Print regexp check of last field ########
                print '        reg = /'.$regexp."/;\n";
                print '        if (!reg.test(newf)) {',"\n";
                if ($text =~ /\$/)
                {
                    $text =~ s/\$//g;
                    print '           alert(newf+"'.$text.'");',"\n";
                }
                else
                {
                    print '           alert("'.$text.'");',"\n";
                }
                print '           return false;',"\n";
                print '        }',"\n";
                print '     }',"\n";
                print "  }\n";
            }
        }
        if ($s =~ /^([\w\s]+)\:([\w\:]+)/)
        {
            #######  Prints a multi field input check where just one  ####
            #######  field input is enough for true                   ####
            my $n = $1;
            my $t = $2;
            my @l = split(/\:/,$t);
            print '  var ok = 0;',"\n";
            for (my $x = 0; $x < @l; $x++)
            {
                print '  txt = f.'.$l[$x].'.value;',"\n";
                print '  if (txt != null && txt.length != 0) {',"\n";
                print '     ok = 1;',"\n";
                print '  }',"\n";
            }
            print '  if (ok == 0) {',"\n";
            print '     alert("One of the \"'.$n.'\" fields must be filled in!");',"\n";
            print '     return false;',"\n";
            print "  }\n";
        }
        if ($s =~ /^([\w\.\-\\\s]+)\,([\w\s\;\"\$\^\\\.\,\*\+]+)/)
        {
           #######  Prints a multi field input check where all  ####
            #######  fields must input for true if one is input  ####
            my $n = $1;
            my $t = $2;
            my @l = split(/\,/,$t);
            print '  var ok = 0;',"\n";
            for (my $x = 0; $x < @l; $x++)
            {
                $l[$x] =~ /^(\w+)/;
                my $st = $1;
                print '  txt = f.'.$st.'.value;',"\n";
                print '  if (txt != null && txt.length != 0) {',"\n";
                print '     ok = 1;',"\n";
                print '  }',"\n";
            }
            print '  if (ok == 1) {',"\n";
            for (my $x = 0; $x < @l; $x++)
            {
                $l[$x] =~ /^(\w+)/;
                my $st = $1;
                print '     txt = f.'.$st.'.value;',"\n";
                print '     if (txt == null || txt.length == 0) {',"\n";
                print '        alert("All of the ',$n,' fields must be filled in if one value is given")',"\n";
                print '        return false;',"\n";
                print '     }',"\n";
                ##### Add regexp check if nescesarry #####
                if ($l[$x] =~ /\;\"([\w\$\^\\\.\+\*]+)\"\;([\$\.\w\s]*)/)
                {
                    my $regexp = $1;
                    my $text = $2;
                    print '     reg = /'.$regexp."/;\n";
                    print '     if (!reg.test(txt)) {',"\n";
                    if ($text =~ /\$/)
                    {
                        $text =~ s/\$//g;
                        print '        alert(txt+"'.$text.'");',"\n";
                    }
                    else
                    {
                        print '        alert("'.$text.'");',"\n";
                    }
                    print '        return false;',"\n";
                    print '     }',"\n";
                }
            }
            print "  }\n";
        }
    }
    print "  return true;\n";
    print "}\n";
    print '</script>';
    print "\n\n";
}

